# copyright 2019-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from mock import patch
from pyramid import testing

from cubicweb import ValidationError, NoResultError
from cubicweb.devtools import BASE_URL, testlib

from cubicweb_saml.utils import (
    build_base_url,
    execute_rql,
    find_group_by_name,
    find_user_by_login,
    generate_hash,
    get_user,
    register_user,
    register_user_into_db,
)


class _SAMLCommonTC(testlib.CubicWebTC):
    def setUp(self):
        super(_SAMLCommonTC, self).setUp()

        self.request = testing.DummyRequest(post={"SAMLResponse": ""})
        self.request.registry["cubicweb.config"] = self.config
        self.request.registry["cubicweb.repository"] = self.repo


class SAMLUtilsTC(_SAMLCommonTC):
    def test_build_base_url(self):
        config = self.vreg.config

        self.assertEqual(build_base_url(config), BASE_URL)

        config["base-url"] = "http://url.without.backslash"

        self.assertEqual(build_base_url(config), "http://url.without.backslash/")

    def test_generate_hash(self):
        self.assertNotEqual(generate_hash("example"), generate_hash("example"))

        self.assertIsNotNone(generate_hash("example", "with", "arguments"))


class SAMLDatabaseTC(_SAMLCommonTC):
    def test_execute_rql(self):
        with self.admin_access.repo_cnx() as cnx:
            self.assertIsNone(
                execute_rql(
                    cnx, "Any U WHERE U is CWUser, U login %(login)s", login="CubicWeb"
                )
            )

            rset = execute_rql(cnx, "Any U WHERE U is CWUser, U login 'admin'")
            self.assertIsInstance(rset, int)
            self.assertEqual(rset, 6)

    def test_find_user_by_login(self):
        with self.admin_access.repo_cnx() as cnx:
            user_eid = register_user_into_db(cnx, "test", "managers", "psswd")

            find_eid = find_user_by_login(cnx, "test")
            self.assertEqual(user_eid, find_eid)

    def test_find_group_by_name(self):
        with self.admin_access.repo_cnx() as cnx:
            self.assertIsNotNone(find_group_by_name(cnx, "managers"))
            self.assertIsNotNone(find_group_by_name(cnx, "users"))
            self.assertIsNone(find_group_by_name(cnx, "watchmen"))

    def test_register_an_user_into_db(self):
        with self.admin_access.repo_cnx() as cnx:
            with self.assertRaises(NoResultError):
                cnx.execute("Any U WHERE U is CWUser, U login 'test'").one()

            user_eid = register_user_into_db(cnx, "test", "managers", "psswd")
            self.assertIsInstance(user_eid, int)

            user = cnx.entity_from_eid(user_eid)
            self.assertEqual(user.login, "test")
            self.assertIn("managers", user.groups)


class SAMLResponseTC(_SAMLCommonTC):
    def test_get_an_unknown_user(self):
        with patch(
            "cubicweb_saml.utils.retrieve_identity_from_client",
            return_value=("unknown_user", {}),
        ):

            self.assertIsNone(get_user(self.request))

    def test_get_an_existing_user(self):
        with self.admin_access.repo_cnx() as cnx:
            user = self.create_user(cnx, "saml_user")

            with patch(
                "cubicweb_saml.utils.retrieve_identity_from_client",
                return_value=("saml_user", {}),
            ):

                userid = get_user(self.request)

                self.assertEqual(userid, user.eid)

    def test_register_a_new_user(self):
        with patch(
            "cubicweb_saml.utils.retrieve_identity_from_client",
            return_value=("saml_user", {}),
        ):

            self.assertIsNone(get_user(self.request))

            userid = register_user(self.request)

            self.assertIsNotNone(userid)
            self.assertEqual(get_user(self.request), userid)

    def test_register_an_existing_user(self):
        with self.admin_access.repo_cnx() as cnx:
            self.create_user(cnx, "saml_user")

            with patch(
                "cubicweb_saml.utils.retrieve_identity_from_client",
                return_value=("saml_user", {}),
            ):

                self.assertIsNotNone(get_user(self.request))

                with self.assertRaises(ValidationError):
                    register_user(self.request)


if __name__ == "__main__":
    from unittest import main

    main()



# copyright 2019-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

options = (
    (
        "saml-allow-unsolicited",
        {
            "type": "yn",
            "default": True,
            "help": "Don't verify that the incoming requests originate from us "
            "via the built-in cache for authn request ids in pysaml2",
            "group": "saml",
            "level": 5,
        },
    ),
    (
        "saml-authn-requests-signed",
        {
            "type": "yn",
            "default": False,
            "help": "Indicates if the Authentication Requests sent by this SP "
            "should be signed by default.",
            "group": "saml",
            "level": 5,
        },
    ),
    (
        "saml-logout-requests-signed",
        {
            "type": "yn",
            "default": True,
            "help": "Indicates if this entity will sign the Logout Requests "
            "originated from it.",
            "group": "saml",
            "level": 5,
        },
    ),
    (
        "saml-want-assertions-signed",
        {
            "type": "yn",
            "default": True,
            "help": "Indicates if this SP wants the IdP to send the assertions "
            "signed. This sets the WantAssertionsSigned attribute of the "
            "SPSSODescriptor node of the metadata so the IdP will know "
            "this SP preference.",
            "group": "saml",
            "level": 5,
        },
    ),
    (
        "saml-want-response-signed",
        {
            "type": "yn",
            "default": False,
            "help": "Indicates that Authentication Responses to this SP must be "
            "signed. If set to True, the SP will not consume any SAML "
            "Responses that are not signed.",
            "group": "saml",
            "level": 5,
        },
    ),
    (
        "saml-register-unknown-user",
        {
            "type": "yn",
            "default": False,
            "help": "Allow to register a new user if this one does not exist in "
            "current database.",
            "group": "saml",
            "level": 5,
        },
    ),
    (
        "saml-register-default-group",
        {
            "type": "string",
            "default": "guests",
            "help": "Set the default group to register new user if the "
            "saml-register-unknown-user option was activated.",
            "group": "saml",
            "level": 5,
        },
    ),
    (
        "saml-register-default-password",
        {
            "type": "string",
            "default": "empty",
            "help": "Set the default password system to use if the "
            "saml-register-unknown-user option was activated. Available "
            "modes: empty (no password), random (set a randomize password "
            "based on hashed string).",
            "group": "saml",
            "level": 5,
        },
    ),
)



# -*- coding: utf-8 -*-
# pylint: disable=W0622
# copyright 2017-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


"""cubicweb-saml application packaging information"""


modname = "cubicweb_saml"
distname = "cubicweb-saml"

numversion = (0, 8, 2)
version = ".".join(str(num) for num in numversion)

license = "LGPL"
author = "LOGILAB S.A. (Paris, FRANCE)"
author_email = "contact@logilab.fr"
description = "SAML2 authentifier"
web = "https://forge.extranet.logilab.fr/cubicweb/cubes/%s" % distname

__depends__ = {
    "cubicweb": ">= 3.38.0, < 3.39.0",
    "logilab-common": ">= 1.4.0",
    "pyramid": ">= 1.10.4",
    "pysaml2": ">= 5.0.0",
    "requests": ">= 2.22.0",
    "zope.interface": ">= 4.1.2",
}
__recommends__ = {}

classifiers = [
    "Environment :: Web Environment",
    "Framework :: CubicWeb",
    "Programming Language :: Python",
]



# copyright 2019-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging

logger = logging.getLogger(__name__)


def includeme(config):
    """Activate the SAML authenticate cube"""
    from cubicweb_saml.pconfig import SAMLAuthenticationPolicy

    if config.registry.get("cubicweb.authpolicy") is None:
        raise ValueError(
            "saml: the default cubicweb auth policy should be "
            "available via the 'cubicweb.authpolicy' registry "
            "config entry"
        )

    cubicweb_sources = config.registry["cubicweb.config"].read_sources_file()

    if "saml" not in cubicweb_sources:
        logger.warning("saml: 'saml' section is missing in cubicweb sources file")

    elif "saml-metadata-uri" not in cubicweb_sources["saml"]:
        logger.warning("saml: 'saml-metadata-uri' option is missing")

    elif not cubicweb_sources["saml"]["saml-metadata-uri"]:
        logger.warning("saml: 'saml-metadata-uri' option is empty")

    else:
        settings = config.get_settings()
        authtkt_prefix = "cubicweb.auth.authtkt.session"
        defaults_settings = {
            "hashalg": settings.get(f"{authtkt_prefix}.hashalg", "sha512"),
            "cookie_name": settings.get(f"{authtkt_prefix}.cookie_name", "auth_tkt"),
            "timeout": settings.get(f"{authtkt_prefix}.timeout", 1200),
            "reissue_time": settings.get(f"{authtkt_prefix}.reissue_time", 120),
            "http_only": settings.get(f"{authtkt_prefix}.http_only", True),
            "secure": settings.get(f"{authtkt_prefix}.secure", True),
        }

        policy = SAMLAuthenticationPolicy(
            settings.get("cubicweb.auth.authtkt.session.secret"),
            settings.get("cubicweb.auth.authtkt.persistent.secret", "notsosecret"),
            defaults=defaults_settings,
        )

        config.registry["cubicweb.authpolicy"]._policies.append(policy)

        config.add_route("saml", "/saml")
        config.scan("cubicweb_saml.pconfig")
